#!/usr/bin/python3

import yamlns
import sys
from io import open # Py2 compat

def _u(x, encoding='utf8'):
	if type(x) == type(b''):
		return x.decode(encoding)
	return type(u'')(x)

def apply(yamlfile, template, output, encoding='utf-8') :
	yaml = yamlns.namespace.load(yamlfile)
	with open(template, encoding=encoding) as f :
		content = f.read()
	result = content.format(**yaml)
	with open(output, 'w', encoding=encoding) as f :
		f.write(_u(result,encoding))

def extract(input_template, output_yaml, encoding='utf-8') :
	with open(input_template, encoding=encoding) as f :
		content = f.read()
	ns = yamlns.namespace.fromTemplateVars(content)
	with open(output_yaml, 'w') as f :
		f.write(_u(ns.dump(), encoding))


def main(args=sys.argv) : # pragma: no cover
	import argparse

	parser = argparse.ArgumentParser(
		description="Takes a template and extract a yaml to fill it.",
		)
	parser.add_argument(
		'--encoding',
		default=None,
		help="forces input encoder for templates",
		dest='encoding',
		)

	sub = parser.add_subparsers(
		dest='subcommand',
		)

	subparser = sub.add_parser(
		'extract',
		help='Collects the template vars from a template and renders an empty yaml file with them',
		)
	subparser.add_argument(
		metavar='input.template',
		dest='input_template',
		)
	subparser.add_argument(
		metavar='output.yaml',
		dest='output_yaml',
		)

	subparser = sub.add_parser(
		'apply',
		help='Apply structured data in a YAML file to a template',
		)
	subparser.add_argument(
		metavar='input.yaml',
		dest='input_yaml',
		)
	subparser.add_argument(
		metavar='template',
		dest='template',
		)
	subparser.add_argument(
		metavar='output',
		dest='output',
		)

	args = parser.parse_args()

	if args.subcommand == 'extract' :
		extract(args.input_template, args.output_yaml, args.encoding)
		return 0

	if args.subcommand == 'apply' :
		apply(args.input_yaml, args.template, args.output, args.encoding)
		return 0

	parser.print_help()
	return -1


if __name__  == '__main__': # pragma: no cover
	sys.exit(main())




# vim: sw=4 ts=4 noet



#!/usr/bin/python3

import unittest
import yamlns
import sys
import io
import os
from . import nstemplate

class NSTemplate_test(unittest.TestCase) :

	def setUp(self) :
		self._toRemove = []

	def toRemove(self, f) :
		self._toRemove.append(f)

	def tearDown(self) :
		for f in self._toRemove :
			if os.path.exists(f):
				os.remove(f)

	def write(self, filename, content):
		with io.open(filename, 'w', encoding='utf8') as f:
			f.write(content)

	def read(self, filename):
		with io.open(filename, encoding='utf8') as f:
			return f.read()

	def test_collectVars_withNoVar(self) :
		content = "booo"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			])

	def test_collectVars_withVar(self) :
		content = "b{boo}o"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'boo',
			])

	def test_collectVars_withManyVarsInALine(self) :
		content = "b{boo}o{far}34"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'boo',
			'far',
			])

	def test_collectVars_multiline(self) :
		content = "b{boo}o\ndfs{nice}"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'boo',
			'nice',
			])

	def test_collectVars_indexinDroped(self) :
		content = "dfs{nice[3]}"
		result = yamlns._collectVars(content)
		self.assertEqual(result,[
			'nice',
			])

	def test_varsTree_plainVars(self) :
		theVars = [
			'boo',
			'far',
			'nice',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'boo: \'\'\n'
			'far: \'\'\n'
			'nice: \'\'\n'
			)

	def test_varsTree_unordered(self) :
		theVars = [
			'nice',
			'far',
			'boo',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'boo: \'\'\n'
			'far: \'\'\n'
			'nice: \'\'\n'
			)

	def test_varsTree_withSubVars(self) :
		theVars = [
			'upper.boo',
			'upper.far',
			'upper.nice',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'upper:\n'
			'  boo: \'\'\n'
			'  far: \'\'\n'
			'  nice: \'\'\n'
			)

	def test_varsTree_withManySubVars(self) :
		theVars = [
			'upper.boo',
			'lower.nice',
			'upper.far',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'lower:\n'
			'  nice: \'\'\n'
			'upper:\n'
			'  boo: \'\'\n'
			'  far: \'\'\n'
			)

	def test_varsTree_withSubSubVars(self) :
		theVars = [
			'upper.lower.boo',
			'upper.lower.far',
			'upper.lower.nice',
			]
		yaml = yamlns._varsTree(theVars).dump()
		self.assertEqual(yaml,
			'upper:\n'
			'  lower:\n'
			'    boo: \'\'\n'
			'    far: \'\'\n'
			'    nice: \'\'\n'
			)

	def test_templateVarsAsYaml(self):
		yaml = yamlns.namespace.fromTemplateVars("""\
			{var1.foo} {var1.lala} {var2}
		""")
		self.assertEqual(yaml.dump(), 
			'var1:\n'
			'  foo: \'\'\n'
			'  lala: \'\'\n'
			'var2: \'\'\n'
		)

	def test_extract(self):
		filemd = 'deleteme.md'
		fileyaml = 'deleteme.yaml'
		self.toRemove(filemd)
		self.toRemove(fileyaml)
		self.write(filemd, u"{var1} {var2.foo}")

		nstemplate.extract(filemd, fileyaml)

		self.assertEqual(self.read(fileyaml),
			u"var1: ''\n"
			u"var2:\n"
			u"  foo: ''\n"
		)

	def test_apply(self):
		filemd = 'deleteme.md'
		fileyaml = 'deleteme.yaml'
		fileout = 'deleteme-output.md'
		self.toRemove(filemd)
		self.toRemove(fileyaml)
		self.toRemove(fileout)

		self.write(filemd, u"{var1} {var2.foo}")
		self.write(fileyaml,
			u"var1: 'value1'\n"
			u"var2:\n"
			u"  foo: 666\n"
		)

		nstemplate.apply(fileyaml, filemd, fileout)

		self.assertEqual(self.read(fileout),
			u"value1 666")





if __name__  == '__main__' :
	sys.exit(unittest.main())




# vim: sw=4 ts=4 noet



#!/usr/bin/python3
#-*- coding: utf-8 -*-

from . import namespace
from . import dateutils
from . import Path
import decimal
import datetime
import math
import unittest
import io
import sys

class Namespace_Test(unittest.TestCase) :

	def setUp(self):
		''

	def tearDown(self):
		import os
		try:
			os.unlink('test.yaml')
		except:
			pass

	def assertContent(self, expected):
		import io
		with io.open("test.yaml", encoding='utf-8') as f:
			result = f.read()
		self.assertEqual(result, expected)

	def writeFile(self, data):
		import io
		with io.open("test.yaml",'w',encoding='utf-8') as f:
			f.write(data)

	def test_construction_default(self):
		ns = namespace()
		self.assertEqual(ns.dump(),
			"{}\n")

	def test_setattribute(self):
		ns = namespace()
		ns.lala = "boo"
		self.assertEqual(ns.lala, 'boo')
		self.assertEqual(ns['lala'], 'boo')
		self.assertEqual(ns.dump(),
			"lala: boo\n")

	def test_setattribute_twice_overwrites(self):
		ns = namespace()
		ns.lala = "boo"
		ns.lala = "foo"
		self.assertEqual(ns.lala, 'foo')
		self.assertEqual(ns['lala'], 'foo')
		self.assertEqual(ns.dump(),
			"lala: foo\n")

	def test_setattribute_many(self):
		ns = namespace()
		ns.lala = "boo"
		ns.lola = "foo"
		self.assertEqual(ns.lola, 'foo')
		self.assertEqual(ns['lola'], 'foo')
		self.assertEqual(ns.dump(),
			"lala: boo\n"
			"lola: foo\n")

	def test_setattribute_differentOrder(self):
		ns = namespace()
		ns.lola = "foo"
		ns.lala = "boo"
		self.assertEqual(ns.dump(),
			"lola: foo\n"
			"lala: boo\n"
			)

	def test_delattribute(self):
		ns = namespace()
		ns.lola = "foo"
		ns.lala = "boo"
		del ns.lola
		self.assertEqual(ns.dump(),
			"lala: boo\n"
			)

	def test_delattribute(self):
		ns = namespace()
		ns.lola = "foo"
		with self.assertRaises(AttributeError):
			del ns.notexisting

	def test_load_None(self):
		self.assertEqual(namespace.loads(""),
			None)

	def test_load_empty(self):
		self.assertEqual(namespace.loads("{}"),
			namespace())

	def test_reload(self):
		yamlcontent = (
			"lala: boo\n"
			"lola: foo\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_reload_inverseOrder(self):
		yamlcontent = (
			"lola: foo\n"
			"lala: boo\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_subnamespaces(self) :
		ns = namespace()
		ns.sub = namespace()
		ns.sub.foo = "value1"
		ns.sub.bar = "value2"
		self.assertEqual(ns.dump(),
			"sub:\n"
			"  foo: value1\n"
			"  bar: value2\n"
			)

	def test_reload_subnamespaces(self) :
		yamlcontent = (
			"sub:\n"
			"  foo: value1\n"
			"  bar: value2\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_loads_fromEncodedUtf8(self):
		yaml = u"hi: caña\n".encode('utf-8')
		result = namespace.loads(yaml)
		self.assertEqual(result,
			namespace(hi=u'caña'))

	def test_loads_unicode(self):
		yaml = u"hi: caña\n"
		result = namespace.loads(yaml)
		self.assertEqual(result,
			namespace(hi=u'caña'))

	# Numbers (int, decimal, float)

	def test_load_decimal(self) :
		yamlcontent = (
			"decimal: 3.41"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.decimal),
			decimal.Decimal)
		self.assertEqual(ns.decimal,
			decimal.Decimal('3.41'))

	def test_load_decimal_negative(self) :
		yamlcontent = (
			"decimal: -3.41"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.decimal),
			decimal.Decimal)
		self.assertEqual(ns.decimal,
			decimal.Decimal('-3.41'))

	def test_load_decimal_infinite(self) :
		ns = namespace.loads(
			"decimal: -.inf"
		)
		self.assertEqual(type(ns.decimal),
			float)
		# Why not decimal?
		self.assertEqual(ns.decimal,
			float("-inf"))

	def test_load_decimal_notANumber(self) :
		ns = namespace.loads(
			"decimal: .nan"
		)
		self.assertEqual(type(ns.decimal),
			float)
		# Why not decimal?
		self.assertTrue(math.isnan(ns.decimal) )

	def test_dump_decimal_inf(self) :
		ns = namespace()
		ns.decimal = decimal.Decimal('-inf')
		self.assertEqual(ns.dump(),
			"decimal: -.inf\n"
		)

	def test_dump_float(self) :
		ns = namespace()
		ns.decimal = 3.41
		self.assertEqual(ns.dump(),
			"decimal: 3.41\n"
		)

	def test_dump_float_inf(self) :

		ns = namespace()
		ns.decimal = float('-inf')
		self.assertEqual(ns.dump(),
			"decimal: -.inf\n"
		)

	def test_dump_decimal(self) :
		ns = namespace()
		ns.decimal = decimal.Decimal('3.41')
		self.assertEqual(ns.dump(),
			"decimal: 3.41\n"
		)

	def test_dump_float_nan(self):
		ns = namespace()
		ns.decimal = float('nan')
		self.assertEqual(ns.dump(),
			"decimal: .nan\n"
		)

	def test_dump_decimal_nan(self):
		ns = namespace()
		ns.decimal = decimal.Decimal('nan')
		self.assertEqual(ns.dump(),
			"decimal: .nan\n"
		)

	# Dates, times

	def test_load_date(self):
		yamlcontent = (
			"adate: 2000-02-28\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.adate),
			dateutils.Date)

	class tzoffset(datetime.tzinfo):
		def __init__(self, name=None, *args, **kwds):
			self._offset = datetime.timedelta(*args, **kwds)
			self._name = name
		def utcoffset(self, dt):
			return self._offset
		def dst(self):
			return datetime.timedelta(0)
		def tzname(self):
			return self._name


	def test_dump_datetime(self):
		yamlcontent = (
			"adate: 2000-02-28 10:20:30.000040+02:00\n"
			)
		ns = namespace()
		ns.adate = datetime.datetime(2000,2,28,10,20,30,40, self.tzoffset(hours=2))
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_dump_datetime_utc(self):
		yamlcontent = (
			"adate: 2000-02-28 10:20:30.000040+00:00\n"
			)
		ns = namespace()
		ns.adate = datetime.datetime(2000,2,28,10,20,30,40, self.tzoffset(hours=0))
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_dump_datetime_naive(self):
		yamlcontent = (
			"adate: 2000-02-28 10:20:30.000040\n"
			)
		ns = namespace()
		ns.adate = datetime.datetime(2000,2,28,10,20,30,40)
		self.assertEqual(ns.dump(),
			yamlcontent)

	def test_load_datetime(self):
		yamlcontent = (
			"adate: 2000-02-28 10:20:30.000040+02:00\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.adate),
			datetime.datetime)
		self.assertEqual(ns.adate,
			datetime.datetime(2000,2,28,10,20,30,40, self.tzoffset(hours=2)))

	def test_load_datetime_utc(self):
		yamlcontent = (
			"adate: 2000-02-28 10:20:30.000040Z\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.adate),
			datetime.datetime)
		self.assertEqual(ns.adate,
			datetime.datetime(2000,2,28,10,20,30,40, self.tzoffset(hours=0)))

	def test_load_datetime_naive(self):
		yamlcontent = (
			"adate: 2000-02-28 10:20:30.000040\n"
			)
		ns = namespace.loads(yamlcontent)
		self.assertEqual(type(ns.adate),
			datetime.datetime)
		self.assertEqual(ns.adate,
			datetime.datetime(2000,2,28,10,20,30,40))

	# dir just for values accessible as attribute

	def test_dir_withNonExistingAttributes(self):
		ns = namespace()
		ns.attr1 = 'value1'
		self.assertTrue('bad' not in dir(ns))

	def test_dir_withExistingAttributes(self):
		ns = namespace()
		ns.attr1 = 'value1'
		self.assertTrue('attr1' in dir(ns))

	def test_dir_withNonIdentifierNames(self):
		ns = namespace()
		ns['a-b'] = 'value1'

		self.assertTrue('a-b' not in dir(ns))

	# multiline strings

	def test_dump_multiline(self):
		ns = namespace()
		ns.text="line\nsecond line"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: |-\n"
			"  line\n"
			"  second line\n"
			"other: 2\n"
		)

	def test_dump_multiline_innnerIndentation(self):
		ns = namespace()
		ns.text="line\n  second line"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: |-\n"
			"  line\n"
			"    second line\n"
			"other: 2\n"
		)

	def test_dump_multiline_firstLineIndentation(self):
		ns = namespace()
		ns.text="  line\nsecond line"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: |2-\n"
			"    line\n"
			"  second line\n"
			"other: 2\n"
		)

	def test_dump_multiline_tabIndentation(self):
		ns = namespace()
		ns.text="\tline\nsecond line"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: \"\\tline\\nsecond line\"\n"
			"other: 2\n"
		)

	def test_dump_multiline_endSpaces(self):
		ns = namespace()
		ns.text="line   \nsecond line"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: \"line   \\nsecond line\"\n"
			"other: 2\n"
		)

	def test_dump_multiline_emptyLine(self):
		ns = namespace()
		ns.text="line\n\nsecond line"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: |-\n"
			"  line\n"
			"\n"
			"  second line\n"
			"other: 2\n"
		)

	def test_dump_multiline_emptyAtEnd(self):
		ns = namespace()
		ns.text="line\nsecond line\n"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: |\n" # this changes, was |-
			"  line\n"
			"  second line\n"
			"other: 2\n"
		)

	def test_dump_multiline_manyEmptyAtEnd(self):
		ns = namespace()
		ns.text="line\nsecond line\n\n\n"
		ns.other=2
		self.assertMultiLineEqual(ns.dump(),
			"text: |+\n" # this changes, was |-
			"  line\n"
			"  second line\n"
			"\n"
			"\n"
			"other: 2\n"
		)

	# Other

	def test_load_recursiveArray(self):
		ns = namespace.loads(
			"list:\n"
			"  - key1: value1\n"
			)
		self.assertEqual(ns,
			namespace(list=[namespace(key1='value1')]))

	# Sources an targets

	def test_load_fromFilename(self):
		self.writeFile(u"hi: caña\n")
		result = namespace.load("test.yaml")
		self.assertEqual(result,
			namespace(hi=u'caña'))

	def test_dump_toFileName(self):
		data = namespace(otra=u'caña')
		data.dump('test.yaml')
		self.assertContent(u"otra: caña\n")


	@unittest.skipIf(not Path, "neither pathlib or pathlib2 not installed")
	def test_load_fromPath(self):
		self.writeFile(u"hi: caña\n")
		result = namespace.load(Path("test.yaml"))
		self.assertEqual(result,
			namespace(hi=u'caña'))

	@unittest.skipIf(not Path, "neither pathlib or pathlib2 not installed")
	def test_dump_toPath(self):
		data = namespace(otra=u'caña')
		data.dump(Path('test.yaml'))
		self.assertContent(u"otra: caña\n")

	@unittest.skipIf(sys.version_info[0]>2,
		"PyYaml in Python 3 requires a text file")
	def test_load_fromBinaryFileObject(self):
		self.writeFile(u"hi: caña\n")
		with io.open('test.yaml', 'rb') as f:
			result = namespace.load(f)
		self.assertEqual(result,
			namespace(hi=u'caña'))

	@unittest.skipIf(sys.version_info[0]>2,
		"PyYaml in Python 3 requires a text file")
	def test_dump_toBinaryFileObject(self):
		data = namespace(otra=u'caña')
		with io.open('test.yaml', 'wb') as f:
			data.dump(f)
		self.assertContent(u"otra: caña\n")

	@unittest.skipIf(sys.version_info[0]==2,
		"PyYaml in Python 2 requires a binary file")
	def test_load_fromTextFileObject(self):
		self.writeFile(u"hi: caña\n")
		with io.open('test.yaml', 'r', encoding='utf8') as f:
			result = namespace.load(f)
		self.assertEqual(result,
			namespace(hi=u'caña'))

	@unittest.skipIf(sys.version_info[0]==2,
		"PyYaml in Python 2 requires a binary file")
	def test_dump_toTextFileObject(self):
		data = namespace(otra=u'caña')
		with io.open('test.yaml','w', encoding='utf8') as f:
			data.dump(f)
		self.assertContent(u"otra: caña\n")


if __name__ == '__main__':
	unittest.main()



# vim: sw=4 ts=4 noet



# -*- encoding: utf8 -*-

import unittest
from . import namespace as ns

class TestUtils_Test(unittest.TestCase):

	from .testutils import assertNsEqual

	def test_assertNsEqual_differentValues(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual(ns(a=3), ns(a=4))
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a: 3\\n' != 'a: 4\\n'\n"
			"- a: 3\n"
			"?    ^\n"
			"+ a: 4\n"
			"?    ^\n"
			)


	def test_assertNsEqual_differentKeys(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual(ns(b=2), ns(c=2))
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'b: 2\\n' != 'c: 2\\n'\n"
			"- b: 2\n"
			"? ^\n"
			"+ c: 2\n"
			"? ^\n"
			)

	def test_assertNsEqual_withOneDict(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual(dict(a=3), ns(a=4))
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a: 3\\n' != 'a: 4\\n'\n"
			"- a: 3\n"
			"?    ^\n"
			"+ a: 4\n"
			"?    ^\n"
			)

	def test_assertNsEqual_withOtherDict(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual(ns(a=3), dict(a=4))
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a: 3\\n' != 'a: 4\\n'\n"
			"- a: 3\n"
			"?    ^\n"
			"+ a: 4\n"
			"?    ^\n"
			)

	def test_assertNsEqual_withOneYaml(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual("a: 3", ns(a=4))
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a: 3\\n' != 'a: 4\\n'\n"
			"- a: 3\n"
			"?    ^\n"
			"+ a: 4\n"
			"?    ^\n"
			)

	def test_assertNsEqual_withOtheYaml(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual(ns(a=3), "a: 4")
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a: 3\\n' != 'a: 4\\n'\n"
			"- a: 3\n"
			"?    ^\n"
			"+ a: 4\n"
			"?    ^\n"
			)

	def test_assertNsEqual_keysGetSorted(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual("a: 3\nb: 4", "b: 4\na: different")
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a: 3\\nb: 4\\n' != 'a: different\\nb: 4\\n'\n"
			"- a: 3\n"
			"+ a: different\n"
			"  b: 4\n"
			)

	def test_assertNsEqual_innerDictsGetSortedToo(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual(
				"a:\n  b: 4\n  c: 3",
				"a:\n  c: 3\n  b: different"
				)
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a:\\n  b: 4\\n  c: 3\\n' != 'a:\\n  b: different\\n  c: 3\\n'\n"
			"  a:\n"
			"-   b: 4\n"
			"+   b: different\n"
			"    c: 3\n"
			)

	def test_assertNsEqual_dictInsideList(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual("a: [{c: 3, b: 2}, 4]", "a: [{b: 2}, 4]")
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'a:\\n- b: 2\\n  c: 3\\n- 4\\n' != 'a:\\n- b: 2\\n- 4\\n'\n"
			"  a:\n"
			"  - b: 2\n"
			"-   c: 3\n"
			"  - 4\n"
			)

	def test_assertNsEqual_lists(self):
		with self.assertRaises(AssertionError) as ctx:
			self.assertNsEqual("[3, 4]", [3, 5])
		self.assertMultiLineEqual(type('u')(ctx.exception),
			"'data:\\n- 3\\n- 4\\n' != 'data:\\n- 3\\n- 5\\n'\n"
			"  data:\n"
			"  - 3\n"
			"- - 4\n"
			"?   ^\n"
			"+ - 5\n"
			"?   ^\n"
			)


# vim: noet ts=4 sw=4



# -*- coding: utf-8 -*-

import unittest
from . import namespace as ns

# Readable verbose testcase listing
unittest.TestCase.__str__ = unittest.TestCase.id

def assertNsEqual(self, dict1, dict2):

    """
    Asserts that both dict have equivalent structure.
    If parameters are strings they are parsed as yaml.
    Comparation by comparing the result of turning them
    to yaml sorting the keys of any dict within the structure.
    """
    def sortedDict(x):
        if type(x) in (dict, ns):
            return ns((k,sortedDict(v)) for k,v in sorted(x.items()))
        if type(x) in (list, tuple):
            return [sortedDict(y) for y in x]
        return x

    def parseIfString(x):
        if type(x) in (type(u''),type(b'')):
            return ns.loads(x)
        return x

    def yaml(x):
        x = parseIfString(x)
        if type(x) in (list, tuple):
            return sortedDict(ns(data=x)).dump()
        return sortedDict(x).dump()

    self.assertMultiLineEqual(
        yaml(dict1),
        yaml(dict2))

# vim: ts=4 sw=4 et
